# -*- coding: utf-8 -*-
from plone.z3cform.fieldsets.interfaces import IDescriptiveGroup
from plone.z3cform.fieldsets.interfaces import IGroupFactory
from z3c.form import group
from zope.interface import implementer


@implementer(IDescriptiveGroup)
class Group(group.Group):
    __name__ = u""
    label = u""
    description = u""
    order = 0

    def getContent(self):
        # Default to sharing content with parent
        return self.__parent__.getContent()


@implementer(IGroupFactory)
class GroupFactory(object):

    def __init__(
        self,
        __name__,
        fields,
        label=None,
        description=None,
        order=None
    ):
        self.__name__ = __name__
        self.fields = fields
        self.label = label or __name__
        self.description = description
        self.order = order

    def __call__(self, context, request, parentForm):
        groupclass = getattr(parentForm, 'group_class', Group)
        group = groupclass(context, request, parentForm)
        group.__name__ = self.__name__
        group.label = self.label
        group.description = self.description
        group.fields = self.fields
        group.order = self.order
        return group



"""A small monkey patch for z3c.form's BaseForm.update() and
GroupForm.update(). We need to call z2.processInputs() before the request is
used, because z3c.form expects them to have been converted to unicode first.
"""
from plone.z3cform.z2 import processInputs
from z3c.form.form import BaseForm
from z3c.form.group import GroupForm

_original_BaseForm_update = BaseForm.update
_original_GroupForm_update = GroupForm.update


def BaseForm_update(self):
    # This monkey patch ensures that processInputs() is called before
    # z3c.form does any work on the request. This is because z3c.form expects
    # charset negotiation to have taken place in the publisher, and will
    # complain about non-unicode strings

    processInputs(self.request)
    _original_BaseForm_update(self)


def GroupForm_update(self):
    # This monkey patch ensures that processInputs() is called before
    # z3c.form does any work on the request. This is because z3c.form expects
    # charset negotiation to have taken place in the publisher, and will
    # complain about non-unicode strings

    processInputs(self.request)
    _original_GroupForm_update(self)


def apply_patch():
    BaseForm.update = BaseForm_update
    GroupForm.update = GroupForm_update



from z3c.form.interfaces import IForm
from zope import schema
from zope.interface import Interface, Attribute
from zope.pagetemplate.interfaces import IPageTemplate


class IFormWrapper(Interface):
    """Form wrapper class.

    This class allows "two-step" rendering, with an outer view rendering
    part of the page and the form class rendering the form area.

    In Zope < 2.12, this is the only way to get z3c.form support, because
    the view class takes care of the acquisition requirement.

    In Zope 2.12 and later, this approach is optional: you may register the
    form class directly as a browser view.
    """

    def update():
        """We use the content provider update/render couple.
        """

    def render():
        """We use the content provider update/render couple.
        """

    form = Attribute("The form class. Should be set at class level")

    form_instance = schema.Object(
        title=u"Instance of the form being rendered",
        description=u"Set by the wrapper code during __init__()",
        readonly=True,
        schema=IForm
    )

    index = schema.Object(
        title=u"Page template instance",
        description=(u"If not set, a template will be found "
                     u"via an adapter lookup"),
        required=False,
        schema=IPageTemplate
    )


class IWrappedForm(Interface):
    """Marker interface applied to wrapped forms during rendering.

    This allows different handling of templates, for example.
    """


class IDeferSecurityCheck(Interface):
    """Marker interface applied to the request during traversal.

    This can be used by other code that wants to skip security
    checks during traversal.
    """



# -*- coding: utf-8 -*-
from plone.testing import Layer
from plone.testing import layered
from plone.testing import z2
from plone.testing import zca
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form.interfaces import IFormLayer
from zope import component
from zope import interface
from zope.component import testing
from zope.configuration import xmlconfig
from zope.publisher.browser import TestRequest as BaseTestRequest

import doctest
import plone.z3cform.templates
import re
import six
import unittest


@interface.implementer(IFormLayer)
class TestRequest(BaseTestRequest):
    pass


def create_eventlog(event=interface.Interface):
    value = []

    @component.adapter(event)
    def log(event):
        value.append(event)

    component.provideHandler(log)
    return value


def setup_defaults():
    # Set up z3c.form defaults
    from z3c.form import browser, button, converter, datamanager, error, field
    from z3c.form import interfaces, validator, widget
    from z3c.form.browser import text
    from zope.pagetemplate.interfaces import IPageTemplate
    import os.path
    import zope.schema

    def getPath(filename):
        return os.path.join(os.path.dirname(browser.__file__), filename)

    component.provideAdapter(validator.SimpleFieldValidator)
    component.provideAdapter(validator.InvariantsValidator)
    component.provideAdapter(datamanager.AttributeField)
    component.provideAdapter(field.FieldWidgets)

    component.provideAdapter(
        text.TextFieldWidget,
        adapts=(zope.schema.interfaces.ITextLine, interfaces.IFormLayer),
    )
    component.provideAdapter(
        text.TextFieldWidget,
        adapts=(zope.schema.interfaces.IInt, interfaces.IFormLayer),
    )

    component.provideAdapter(
        widget.WidgetTemplateFactory(getPath('text_input.pt'), 'text/html'),
        (None, None, None, None, interfaces.ITextWidget),
        IPageTemplate,
        name=interfaces.INPUT_MODE,
    )
    component.provideAdapter(
        widget.WidgetTemplateFactory(getPath('text_display.pt'), 'text/html'),
        (None, None, None, None, interfaces.ITextWidget),
        IPageTemplate,
        name=interfaces.DISPLAY_MODE,
    )

    component.provideAdapter(
        widget.WidgetTemplateFactory(
            getPath('checkbox_input.pt'), 'text/html'
        ),
        (None, None, None, None, interfaces.ICheckBoxWidget),
        IPageTemplate,
        name=interfaces.INPUT_MODE,
    )
    component.provideAdapter(
        widget.WidgetTemplateFactory(
            getPath('checkbox_display.pt'), 'text/html'
        ),
        (None, None, None, None, interfaces.ICheckBoxWidget),
        IPageTemplate,
        name=interfaces.DISPLAY_MODE,
    )
    # Submit Field Widget
    component.provideAdapter(
        widget.WidgetTemplateFactory(getPath('submit_input.pt'), 'text/html'),
        (None, None, None, None, interfaces.ISubmitWidget),
        IPageTemplate,
        name=interfaces.INPUT_MODE,
    )

    component.provideAdapter(converter.FieldDataConverter)
    component.provideAdapter(converter.FieldWidgetDataConverter)
    component.provideAdapter(
        button.ButtonAction, provides=interfaces.IButtonAction
    )
    component.provideAdapter(button.ButtonActions)
    component.provideAdapter(button.ButtonActionHandler)
    component.provideAdapter(error.StandardErrorViewTemplate)

    # Make traversal work; register both the default traversable
    # adapter and the ++view++ namespace adapter
    component.provideAdapter(
        zope.traversing.adapters.DefaultTraversable, [None]
    )
    component.provideAdapter(
        zope.traversing.namespace.view, (None, None), name='view'
    )

    # Setup ploneform macros, simlulating the ZCML directive
    plone.z3cform.templates.Macros.index = ViewPageTemplateFile(
        plone.z3cform.templates.path('macros.pt')
    )

    component.provideAdapter(
        plone.z3cform.templates.Macros,
        (None, None),
        zope.publisher.interfaces.browser.IBrowserView,
        name='ploneform-macros',
    )

    # setup plone.z3cform templates
    from zope.pagetemplate.interfaces import IPageTemplate

    component.provideAdapter(
        plone.z3cform.templates.wrapped_form_factory,
        (None, None),
        IPageTemplate,
    )

    from z3c.form.interfaces import IErrorViewSnippet

    component.provideAdapter(
        error.ErrorViewSnippet,
        (None, None, None, None, None, None),
        IErrorViewSnippet,
    )


class P3FLayer(Layer):
    defaultBases = (z2.STARTUP,)

    def setUp(self):
        self['configurationContext'] = context = zca.stackConfigurationContext(
            self.get('configurationContext')
        )
        import plone.z3cform

        xmlconfig.file('testing.zcml', plone.z3cform, context=context)
        import z3c.form

        xmlconfig.file('configure.zcml', z3c.form, context=context)

    def tearDown(self):
        del self['configurationContext']


P3F_FIXTURE = P3FLayer()
FUNCTIONAL_TESTING = z2.FunctionalTesting(
    bases=(P3F_FIXTURE,), name="plone.z3cform:Functional"
)


class Py23DocChecker(doctest.OutputChecker):
    def check_output(self, want, got, optionflags):
        if six.PY2:
            got = re.sub(
                'LocationError', 'zope.location.interfaces.LocationError', got
            )
            got = re.sub("u'(.*?)'", "'\\1'", got)
        return doctest.OutputChecker.check_output(self, want, got, optionflags)


def test_suite():
    layout_txt = layered(
        doctest.DocFileSuite('layout.rst', checker=Py23DocChecker()),
        layer=FUNCTIONAL_TESTING,
    )
    inputs_txt = layered(
        doctest.DocFileSuite('inputs.txt', checker=Py23DocChecker()),
        layer=FUNCTIONAL_TESTING,
    )
    fieldsets_txt = layered(
        doctest.DocFileSuite('fieldsets/README.rst', checker=Py23DocChecker()),
        layer=FUNCTIONAL_TESTING,
    )

    traversal_txt = layered(
        doctest.DocFileSuite('traversal.txt', checker=Py23DocChecker()),
        layer=FUNCTIONAL_TESTING,
    )
    crud_readme_txt = layered(
        doctest.DocFileSuite('crud/README.txt', checker=Py23DocChecker()),
        layer=zca.UNIT_TESTING,
    )
    crud_py = layered(
        doctest.DocTestSuite(
            'plone.z3cform.crud.crud',
            setUp=testing.setUp,
            tearDown=testing.tearDown,
            checker=Py23DocChecker(),
        ),
        layer=zca.UNIT_TESTING,
    )
    traversal_txt = layered(
        doctest.DocFileSuite('traversal.txt', checker=Py23DocChecker()),
        layer=FUNCTIONAL_TESTING,
    )

    return unittest.TestSuite(
        [
            layout_txt,
            inputs_txt,
            fieldsets_txt,
            traversal_txt,
            crud_readme_txt,
            crud_py,
        ]
    )



import ZPublisher.HTTPRequest
import cgi
import z3c.form.converter
import z3c.form.interfaces
import zope.publisher.browser


class FileUploadDataConverter(z3c.form.converter.FileUploadDataConverter):
    """Although ZPublisher's and zope.publisher's FileUpload
    implementations are almost identical, ``FileUploadDataConverter``
    makes an ``isinstance`` call that breaks duck-typing.

    Therefore, we override the stock ``FileUploadDataConverter`` with
    this little class that will do the right thing when a Zope 2
    FileUpload object is received.
    """

    def toFieldValue(self, value):
        """See interfaces.IDataConverter"""
        if isinstance(value, ZPublisher.HTTPRequest.FileUpload):
            fieldstorage = cgi.FieldStorage()
            fieldstorage.file = value
            fieldstorage.headers = value.headers
            fieldstorage.filename = value.filename
            value = zope.publisher.browser.FileUpload(fieldstorage)

        return super(FileUploadDataConverter, self).toFieldValue(value)



from zope.schema import vocabulary
import z3c.form.browser.checkbox
import z3c.form.interfaces
import z3c.form.term
import zope.component
import zope.interface
import zope.schema.interfaces


class SingleCheckBoxWidget(z3c.form.browser.checkbox.SingleCheckBoxWidget):

    def update(self):
        self.ignoreContext = True
        super(SingleCheckBoxWidget, self).update()

    def updateTerms(self):
        # The default implementation would render "selected" as a
        # lebel for the single checkbox.  We use no label instead.
        if self.terms is None:
            self.terms = z3c.form.term.Terms()
            self.terms.terms = vocabulary.SimpleVocabulary((
                vocabulary.SimpleTerm(True, 'selected', u''),
            ))
        return self.terms

    def extract(self, default=z3c.form.interfaces.NOVALUE, setErrors=True):
        # The default implementation returns [] here.
        if (self.name not in self.request and
                self.name + '-empty-marker' in self.request):
            return default
        else:
            try:
                return super(
                    SingleCheckBoxWidget,
                    self).extract(
                    default,
                    setErrors=setErrors)
            except TypeError:
                # for z3c.form <= 1.9.0
                return super(SingleCheckBoxWidget, self).extract(default)


@zope.component.adapter(
    zope.schema.interfaces.IBool,
    z3c.form.interfaces.IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def SingleCheckBoxFieldWidget(field, request):
    return z3c.form.widget.FieldWidget(field, SingleCheckBoxWidget(request))

# BBB:
singlecheckboxwidget_factory = SingleCheckBoxFieldWidget



# -*- coding: utf-8 -*-
from ZPublisher import HTTPRequest
from zope import interface
from zope.i18n.interfaces import IUserPreferredCharsets
from zope.publisher.browser import isCGI_NAME
from zope.publisher.interfaces.browser import IBrowserApplicationRequest

import six
import z3c.form.interfaces


class IFixedUpRequest(interface.Interface):
    """Marker interface used to ensure we don't fix up the request twice
    """


class IProcessedRequest(interface.Interface):
    """Marker interface used to ensure we don't process the request inputs
    twice.
    """

# Safer versions of the functions in Products.Five.browser.decode


def processInputs(request, charsets=None):
    """Process the values in request.form to decode strings to unicode, using
    the passed-in list of charsets. If none are passed in, look up the user's
    preferred charsets. The default is to use utf-8.
    """

    if IProcessedRequest.providedBy(request):
        return

    if charsets is None:
        envadapter = IUserPreferredCharsets(request, None)
        if envadapter is None:
            charsets = ['utf-8']
        else:
            charsets = envadapter.getPreferredCharsets() or ['utf-8']

    for name, value in request.form.items():
        if not (isCGI_NAME(name) or name.startswith('HTTP_')):
            if isinstance(value, six.binary_type):
                request.form[name] = _decode(value, charsets)
            elif isinstance(value, (list, tuple,)):
                newValue = []
                for val in value:
                    if isinstance(val, six.binary_type):
                        val = _decode(val, charsets)
                    newValue.append(val)

                if isinstance(value, tuple):
                    newValue = tuple(value)

                request.form[name] = newValue
            elif isinstance(value, HTTPRequest.record):
                newValue = {}
                for key, val in value.items():
                    newValue[key] = _decode(val, charsets)
                request.form[name] = newValue

    interface.alsoProvides(request, IProcessedRequest)


def _decode(text, charsets):
    for charset in charsets:
        try:
            # decode recursively
            return HTTPRequest._decode(text, charset)
        except (UnicodeError, AttributeError):
            pass
    return text


def switch_on(view, request_layer=z3c.form.interfaces.IFormLayer):
    """Fix up the request and apply the given layer. This is mainly useful
    in Zope < 2.10 when using a wrapper layout view.
    """

    request = view.request

    if (
        not IFixedUpRequest.providedBy(request) and
        not IBrowserApplicationRequest.providedBy(request)
    ):
        interface.alsoProvides(request, IFixedUpRequest)
        interface.alsoProvides(request, request_layer)



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: __init__.py 97 2007-03-29 22:58:27Z rineichen $
"""
__docformat__ = "reStructuredText"

import six
import zope.component
import zope.interface
import zope.schema.interfaces


try:
    # z3c.form 2.0 or later
    from z3c.form.browser.textlines import TextLinesFieldWidget
    from z3c.form.browser.textlines import TextLinesFieldWidgetFactory
    from z3c.form.browser.textlines import TextLinesWidget
    from z3c.form.converter import TextLinesConverter
    from z3c.form.interfaces import ITextLinesWidget

except ImportError:

    # backport for z3c.form 1.9

    from z3c.form import interfaces
    from z3c.form import widget
    from z3c.form import converter
    from z3c.form.browser import textarea

    class ITextLinesWidget(interfaces.IWidget):
        """Text lines widget."""

    class TextLinesWidget(textarea.TextAreaWidget):
        """Input type sequence widget implementation."""
        zope.interface.implementsOnly(ITextLinesWidget)

    def TextLinesFieldWidget(field, request):
        """IFieldWidget factory for TextLinesWidget."""
        return widget.FieldWidget(field, TextLinesWidget(request))

    @zope.interface.implementer(interfaces.IFieldWidget)
    def TextLinesFieldWidgetFactory(field, value_type, request):
        """IFieldWidget factory for TextLinesWidget."""
        return TextLinesFieldWidget(field, request)

    @zope.component.adapter(
        zope.schema.interfaces.ISequence,
        ITextLinesWidget,
    )
    class TextLinesConverter(converter.BaseDataConverter):
        """Data converter for ITextLinesWidget."""

        def toWidgetValue(self, value):
            """Convert from text lines to HTML representation."""
            # if the value is the missing value, then an empty list is
            # produced.
            if value is self.field.missing_value:
                return u''
            return u'\n'.join(six.text_type(v) for v in value)

        def toFieldValue(self, value):
            """See interfaces.IDataConverter"""
            collectionType = self.field._type
            if isinstance(collectionType, tuple):
                collectionType = collectionType[-1]
            if not len(value):
                return self.field.missing_value
            valueType = self.field.value_type._type
            if isinstance(valueType, tuple):
                valueType = valueType[0]
            return collectionType(valueType(v) for v in value.split())

# additional


@zope.component.adapter(
    zope.schema.interfaces.ISet,
    ITextLinesWidget,
)
class TextLinesSetConverter(TextLinesConverter):
    """Data converter for ITextLinesWidget operating on a set."""

    def toWidgetValue(self, value):
        """Convert from text lines to HTML representation."""
        # if the value is the missing value, then an empty list is produced.
        if value is self.field.missing_value:
            return u''
        return u'\n'.join(six.text_type(v) for v in sorted(value))


@zope.component.adapter(
    zope.schema.interfaces.IFrozenSet,
    ITextLinesWidget,
)
class TextLinesFrozenSetConverter(TextLinesConverter):
    """Data converter for ITextLinesWidget operating on a frozenset."""

    def toWidgetValue(self, value):
        """Convert from text lines to HTML representation."""
        # if the value is the missing value, then an empty list is produced.
        if value is self.field.missing_value:
            return u''
        return u'\n'.join(six.text_type(v) for v in sorted(value))



# -*- coding: utf-8 -*-
from plone.z3cform import MessageFactory as _
from plone.z3cform.fieldsets import utils
from plone.z3cform.fieldsets.interfaces import IExtensibleForm
from plone.z3cform.fieldsets.interfaces import IFormExtender
from z3c.form.group import GroupForm
from zope.component import getAdapters
from zope.interface import implementer


def order_key(adapter_tuple):
    return adapter_tuple[1].order


@implementer(IFormExtender)
class FormExtender(object):
    """Base class for IFormExtender adapters with convenience APIs
    """

    # Change this to prioritise
    order = 0

    def __init__(self, context, request, form):
        self.context = context
        self.request = request
        self.form = form

    def add(self, *args, **kwargs):
        """Add one or more fields. Keyword argument 'index' can be used to
        specify an index to insert at. Keyword argument 'group' can be used
        to specify the label of a group, which will be found and used or
        created if it doesn't exist.
        """

        utils.add(self.form, *args, **kwargs)

    def remove(self, field_name, prefix=None):
        """Get rid of a field. The omitted field will be returned.
        """

        return utils.remove(self.form, field_name, prefix=prefix)

    def move(
            self,
            field_name,
            before=None,
            after=None,
            prefix=None,
            relative_prefix=None):
        """Move the field with the given name before or after another field.
        """

        utils.move(self.form, field_name, before=before, after=after,
                   prefix=prefix, relative_prefix=relative_prefix)


@implementer(IExtensibleForm)
class ExtensibleForm(GroupForm):

    groups = []
    default_fieldset_label = _(u"Default")

    def update(self):
        self.updateFields()
        super(ExtensibleForm, self).update()

    def updateFields(self):
        extenders = getAdapters(
            (self.context, self.request, self),
            IFormExtender
        )
        for name, extender in sorted(extenders, key=order_key):
            extender.update()



from Acquisition import aq_base
from Acquisition import aq_inner
from plone.z3cform import z2
from plone.z3cform.interfaces import IDeferSecurityCheck
from plone.z3cform.interfaces import IFormWrapper
from z3c.form import util
from z3c.form.interfaces import IForm
from zope.component import adapter
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.interface import noLongerProvides
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.traversing.interfaces import ITraversable
from zope.traversing.interfaces import TraversalError


@adapter(IForm, IBrowserRequest)
@implementer(ITraversable)
class FormWidgetTraversal(object):
    """Allow traversal to widgets via the ++widget++ namespace. The context
    is the from itself (used when the layout wrapper view is not used).

    Note that to support security in Zope 2.10, the widget being traversed to
    must have an __of__ method, i.e. it must support acquisition. The easiest
    way to do that, is to mix in Acquisition.Explicit. The acquisition parent
    will be the layout form wrapper view.

    In Zope 2.12, this is not necessary, because we also set the __parent__
    pointer of the returned widget to be the traversal context.

    Unfortunately, if you mix in Acquisition.Explicit in Zope 2.12 *and* the
    class implements IAcquirer, Zope may complain because the view probably
    does *not* implement acquisition (in Zope 2.12, views no longer mix in
    Acquisiton.Explicit). To support both Zope 2.10 and Zope 2.12, you will
    need to cheat and mix in Acquisition.Explicit, but use implementsOnly()
    or some other mechanism to make sure the instance does not provide
    IAcquirer.
    """

    def __init__(self, context, request=None):
        self.context = context
        self.request = request

    def _prepareForm(self):
        return self.context

    def traverse(self, name, ignored):

        form = self._prepareForm()

        # Since we cannot check security during traversal,
        # we delegate the check to the widget view.
        alsoProvides(self.request, IDeferSecurityCheck)
        form.update()
        noLongerProvides(self.request, IDeferSecurityCheck)

        # If name begins with form.widgets., remove it
        form_widgets_prefix = util.expandPrefix(
            form.prefix) + util.expandPrefix(form.widgets.prefix)
        if name.startswith(form_widgets_prefix):
            name = name[len(form_widgets_prefix):]

        # Split string up into dotted segments and work through
        target = aq_base(form)
        parts = name.split('.')
        while len(parts) > 0:
            part = parts.pop(0)
            # i.e. a z3c.form.widget.MultiWidget
            if isinstance(getattr(target, 'widgets', None), list):
                try:
                    # part should be integer index in list, look it up
                    target = target.widgets[int(part)]
                except IndexError:
                    raise TraversalError("'" + part + "' not in range")
                except ValueError:
                    # HACK: part isn't integer. Iterate through widgets to
                    # find matching name. This is required for
                    # DataGridField, which appends 'AA' and 'TT' rows to
                    # it's widget list.
                    full_name = util.expandPrefix(target.prefix) + part
                    filtered = [w for w in target.widgets
                                if w.name == full_name]
                    if len(filtered) == 1:
                        target = filtered[0]
                    else:
                        raise TraversalError("'" + part + "' not valid index")
            elif hasattr(target, 'widgets'):  # Either base form, or subform
                # Check to see if we can find a "Behaviour.widget"
                new_target = None
                if len(parts) > 0:
                    new_target = self._form_traverse(
                        target,
                        part +
                        '.' +
                        parts[0])

                if new_target is not None:
                    # Remove widget name from stack too
                    parts.pop(0)
                else:
                    # Find widget in form without behaviour prefix
                    new_target = self._form_traverse(target, part)

                target = new_target
            # subform-containing widget, only option is to go into subform
            elif hasattr(target, 'subform'):
                if part == 'widgets':
                    target = target.subform
                else:
                    target = None
            else:
                raise TraversalError(
                    'Cannot traverse through ' +
                    target.__repr__())

            # Could not traverse from target to part
            if target is None:
                raise TraversalError(part)

        # Make the parent of the widget the traversal parent.
        # This is required for security to work in Zope 2.12
        if target is not None:
            target.__parent__ = aq_inner(self.context)
            return target
        raise TraversalError(name)

    # Look for name within a form
    def _form_traverse(self, form, name):
        if name in form.widgets:
            return form.widgets.get(name)
        # If there are no groups, give up now
        if getattr(aq_base(form), 'groups', None) is None:
            return None
        for group in form.groups:
            if group.widgets and name in group.widgets:
                return group.widgets.get(name)


@adapter(IFormWrapper, IBrowserRequest)
class WrapperWidgetTraversal(FormWidgetTraversal):
    """Allow traversal to widgets via the ++widget++ namespace. The context
    is the from layout wrapper.

    The caveat about security above still applies!
    """

    def _prepareForm(self):
        form = self.context.form_instance
        z2.switch_on(self.context, request_layer=self.context.request_layer)
        return form



from Acquisition import aq_inner
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import BoundPageTemplate
from plone.z3cform import interfaces
from plone.z3cform import z2
from zope.pagetemplate.interfaces import IPageTemplate
import z3c.form.interfaces
import zope.component
import zope.interface


@zope.interface.implementer(interfaces.IFormWrapper)
class FormWrapper(BrowserView):
    """Use this as a base class for your Five view and override the
    'form' attribute with your z3c.form form class.  Your form will
    then be rendered in the contents area of a layout template, the
    'index' attribute.

    Use the 'wrap' function in this module if you don't like defining
    classes.
    """

    form = None  # override this with a form class.
    index = None  # override with a page template, or rely on an adapter
    request_layer = z3c.form.interfaces.IFormLayer

    def __init__(self, context, request):
        super(FormWrapper, self).__init__(context, request)
        if self.form is not None:
            self.form_instance = self.form(
                aq_inner(self.context), self.request)
            self.form_instance.__name__ = self.__name__

    def update(self):
        """On update, we switch on the zope3 request, needed to work with
        our z3c form. We update here the wrapped form.

        Override this method if you have more than one form.
        """

        if not z3c.form.interfaces.ISubForm.providedBy(self.form_instance):
            zope.interface.alsoProvides(
                self.form_instance,
                interfaces.IWrappedForm)

        z2.switch_on(self, request_layer=self.request_layer)
        self.form_instance.update()

        # If a form action redirected, don't render the wrapped form
        if self.request.response.getStatus() in (302, 303):
            self.contents = ""
            return

        # A z3c.form.form.AddForm does a redirect in its render method.
        # So we have to render the form to see if we have a redirection.
        # In the case of redirection, we don't render the layout at all.
        self.contents = self.form_instance.render()

    def __call__(self):
        """We use the update/render pattern. If a redirect happens in the
        meantime, we simply skip the rendering.
        """
        self.update()
        return self.render()

    def render(self):
        """This method renders the outer skeleton template, which in
        turn calls the 'contents' method below.

        We use an indirection to 'self.index' here to allow users to
        override the skeleton template through the 'browser' zcml
        directive. If no index template is set, we look up a an adapter from
        (self, request) to IPageTemplate and use that instead.
        """
        if self.request.response.getStatus() in (302, 303):
            return u""
        if self.index is None:
            template = zope.component.getMultiAdapter(
                (self, self.request), IPageTemplate)
            return BoundPageTemplate(template, self)()
        return self.index()

    def label(self):
        """Override this method to use a different way of acquiring a
        label or title for your page.  Overriding this with a simple
        attribute works as well.
        """
        return self.form_instance.label


def wrap_form(form, __wrapper_class=FormWrapper, **kwargs):
    class MyFormWrapper(__wrapper_class):
        pass
    assert z3c.form.interfaces.IForm.implementedBy(form)
    MyFormWrapper.form = form
    for name, value in kwargs.items():
        setattr(MyFormWrapper, name, value)
    return MyFormWrapper



from setuptools import setup, find_packages

import os

__version__ = '1.1.2'


def description():
    join = lambda *paths: os.path.join('src', 'plone', 'z3cform', *paths)
    return (
        open('README.rst').read()
        + '\n'
        + open(join('fieldsets', 'README.rst')).read()
        + '\n'
        + open(join('crud', 'README.txt')).read()
        + '\n'
        + open('CHANGES.rst').read()
        + '\n'
    )


setup(
    name='plone.z3cform',
    version=__version__,
    description='plone.z3cform is a library that allows use of z3c.form '
    'with Zope and the CMF.',
    long_description=description(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Plone',
        'Framework :: Plone :: 5.2',
        'Framework :: Plone :: Core',
        'Framework :: Zope :: 4',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='Plone CMF Python Zope CMS Webapplication',
    author='Plone Foundation',
    author_email='releasemanager@plone.org',
    url='https://github.com/plone/plone.z3cform',
    license='GPL version 2',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['plone'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'plone.batching',
        'six',
        'z3c.form',
        'zope.i18n>=3.4',
        'zope.browserpage',
        'zope.component',
        'Zope',
    ],
    extras_require={'test': ['lxml', 'plone.testing[z2]']},
)



"""This module provides "form template factories" that can be
registered to provide default form templates for forms and subforms.

The default templates draw from a macro page template which you can use by
itself to render parts of it.
"""
from Acquisition import IAcquirer
from Acquisition import ImplicitAcquisitionWrapper
from Products.Five.browser.pagetemplatefile import (
    ViewPageTemplateFile as ZopeTwoPageTemplateFile
)
from z3c.form import util
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.pagetemplate.interfaces import IPageTemplate

import os
import plone.z3cform
import plone.z3cform.layout
import z3c.form.form
import z3c.form.interfaces
import z3c.form.widget
import zope.publisher.browser


def path(filename):
    return os.path.join(
        os.path.dirname(plone.z3cform.__file__), 'pagetemplates', filename
    )


# Zope 2-compatible form and widget template factory classes.


class FormTemplateFactory(z3c.form.form.FormTemplateFactory):
    """Form template factory that will use Chameleon if installed (via
    five.pt), or the Zope 2 ViewPageTemplateFile from Products.Five if not.

    You can use this for a wrapped form, but not for a form that is going
    to be rendered as a standalone view. Use ZopeTwoFormTemplateFactory for
    that instead.
    """

    def __init__(
        self, filename, contentType='text/html', form=None, request=None
    ):
        self.template = ViewPageTemplateFile(
            filename, content_type=contentType
        )
        zope.component.adapter(
            util.getSpecification(form), util.getSpecification(request)
        )(self)
        zope.interface.implementer(IPageTemplate)(self)


class ZopeTwoFormTemplateFactory(z3c.form.form.FormTemplateFactory):
    """Form template factory for Zope 2 page templates.

    Use this for any form which is going to be rendered as a view, or any
    form wrapper view.
    """

    def __init__(
        self, filename, contentType='text/html', form=None, request=None
    ):
        self.template = ZopeTwoPageTemplateFile(
            filename, content_type=contentType
        )
        zope.component.adapter(
            util.getSpecification(form), util.getSpecification(request)
        )(self)
        zope.interface.implementer(IPageTemplate)(self)

    def __call__(self, form, request):
        template = self.template
        if IAcquirer.providedBy(template):
            return ImplicitAcquisitionWrapper(template, form)
        return template


class ZopeTwoWidgetTemplateFactory(z3c.form.widget.WidgetTemplateFactory):
    """A variant of z3c.form's widget.WidgetTemplateFactory which uses Zope 2
    page templates. This should only be necessary if you strictly need the
    extra Zope 2-isms of Five's ViewPageTemplateFile.
    """

    def __init__(
        self,
        filename,
        contentType='text/html',
        context=None,
        request=None,
        view=None,
        field=None,
        widget=None,
    ):
        self.template = ViewPageTemplateFile(
            filename, content_type=contentType
        )
        zope.component.adapter(
            util.getSpecification(context),
            util.getSpecification(request),
            util.getSpecification(view),
            util.getSpecification(field),
            util.getSpecification(widget),
        )(self)
        zope.interface.implementer(IPageTemplate)(self)


# View containing common macros


class Macros(zope.publisher.browser.BrowserView):
    def __getitem__(self, key):
        return self.index.macros[key]


# Default templates for the wrapped layout view use case

layout_factory = ZopeTwoFormTemplateFactory(
    path('layout.pt'), form=plone.z3cform.interfaces.IFormWrapper
)

wrapped_form_factory = FormTemplateFactory(
    path('wrappedform.pt'), form=plone.z3cform.interfaces.IWrappedForm
)

# Default templates for the standalone form use case

standalone_form_factory = ZopeTwoFormTemplateFactory(
    path('form.pt'), form=z3c.form.interfaces.IForm
)

# Default templates for subforms

subform_factory = FormTemplateFactory(
    path('subform.pt'), form=z3c.form.interfaces.ISubForm
)
